; SolarTimer3b

;THE LAST CHIP BURN WAS ON Sept. 14, 2013 for a COMMON CATHODE type LED
;    (The older, first version of Solar Timer 3)

;SPECIAL NOTE 1:  This file was modified from the original SolarTimer3.
;   It has now been modified to reflect SolarTimer3b coding.
;   The old SolarTime3 coding is only available on the simister.net
;   or Tripod Web sites.  

;SPECIAL NOT 2:  This circuit has lines which can be activated for common cathode or 
;	common anode type bi-color LED.  These lines can be turned on or off.
;ALTER these lines:  112, 981-987, 1022-1028, 1041-1047, 1444-1447. 1470, 1485, 1508, 
;   1514, 1523-1529, 1553-1559, 1568-1573, 1604-1610, 1734-1740, 1746,1752, 1756
;   	

; A second Solar Timer 3b was constructed on September 4, 2013.   Generally, no
;   changes to coding were made from the previous Solar Timer 3 project. EXCEPT 
;   for BiColor LED.  Instead of common cathode, a common anode was used for
;   this project.

;Crystal frequency = 32,768 Hz

;July 18, 2013
;Text note update - Sept. 4, 2013
;This project uses common anode on LEDs

;Copyright 2012 by:

;Wayne Simister
;Automated Sound
;584 Cortez Street
;Salt Lake City, UT 84103
;wsimister@hotmail.com


;INSTRUCTIONS:

;DIT = 1/8th second touch    DAH = 1/2 to 3/4 second touch     
;HOLD DOWN = wait until RED LED flashes

;Default is a clock which flashes out the time.  First, day of week, followed by hours,
;  then ten minute segments (double flashes), and finally single minutes.  

;A hold down, dah, and dit combination enters a time for the timer.  Hold down pushes
;  program one hour for each flash of the LED.  A DAH adds 10 minutes to the total time.
;  A DIT will add single minutes to the total time.  It is important that Hold Down, DAH,
;  and DIT bit be entered in that order.

;While the timer is running, a single DIT push will beep out the remaining time left.  Two
;  pushes will abort the timing cycle.

;After time interval has finished, a single push (DIT) will exit the timer mode.  TWO pushes
;   (DITS) will repeat the last entered time cycle.

;Other function are as follows:

;  DIT, DAH, DIT, DIT, DIT = Toggles the alarm relay on and off. A short beep indicates ON
;     A long beep indicates OFF
;  DIT, DAH, DIT, DIT, DAH = Toggles the alarm on and off.
;  DIT, DAH, DIT, DAH, DIT = Set alarm time in 24 hour format
;     Two entries are needed.  First is hours (plus one), second is minutes (plus one)
;  DIT, DAH, DIT, DAH,DAH = Sets the clock.  Four entries are needed.  First is Day of Week,
;     Second is hour, Third is minutes.  Fourth starts the clock running
;  DIT, DAH, DAH, DIT, DIT = Sets clock correction.  Five entries are required. First is 16,777,216
;     seconds entry (0-255). Second is 65,536 seconds entry (0-255).  Third is 256 
;     seconds entry (0-255).  Fourth is individual seconds entry (0-255).  The last entry
;     is for clock correction:  (1 push = NO clock correction, 2 pushes = subtract one 
;     second,  3 pushes will add one second. 
;	DIT, DAH, DAH, DIT, DAH = Quick Alarm Set Entry.  One entry.  Enter number of quarter
;     hours past midnight (Plus One). 
;   DIT, DAH, DAH, DAH, DAH = Toggles Beep on and off for ALARM only (Does not
;     affect TIMER operations).  (1) = ON,  (2) = OFF

;Following fuses are set on ATTINY2313V chip:

;  Preserve EEPROM memory through the chip erase cycle

;  Ext Low Frequency Xtl/Resonator 1001 SUT 01. 

;  When burning chip, All programming must be done at board's fast chip clock speed,
;      When testing, reduce speed to 37.7 KHz speed. 


;UniversalDelays section: If r19,0(1) flag is set, and button is pushed, all delays are
;   immediately cancelled (skipped over)


.include "tn2313def.inc"


;*****Initialization


rjmp	START1
rjmp	START1
rjmp	START1
rjmp	START1
rjmp	ClockCntDwnInt					

	
START1:			;set up stack pointer and initialize cold start

ldi r16,low(RAMEND)	; Load low byte address of end of RAM into register R16
out	SPL,R16			; Initialize stack pointer to end of internal RAM

ser	r16	  		;Set to all ones for out port direction as output
out	DDRB,r16 	;Set PORTB to output (actually, only 0-5 used)

ldi	r16,251
out	DDRD,r16  ;Set all Port D to output --- except port 2(4) which is an input


clr	r16		
out	PortD,r16	;Off all port D out lines

ldi	r16,8	;For common cathod bi-color LED

;ldi	r16,14		;high for bits 2,3,4 (Sonalert off, LEDs off)
out	PortB,r16


in	r16,TIMSK  ;enable 16-bit match 
ori	r16,64	   ;set bit 4(16) on -- of TIMSK (timer 1 compare register A)         		
out	TIMSK,r16

in	r16,TCCR1B
ori	r16,9		; No prescaler (1).  Zero upon match 
				;  zero counter upon match of A (bit 3(8))
out	TCCR1B,r16                                      

;------------------------

ldi	r16,128
out	OCR1AH,r16  ; Freuqency assumed to be 32,768

ldi	r16,0
out	OCR1AL,r16  ;loads the compare register A for timer 1 (1 second)

;------------------------

clr	r19			;clear flags register, R19
clr	r20			;clear flags register, R20
sts	$006B,r19	;clear  seconds counter

;-----------------------

;Reads in clock correction data from EEPROM - r16 location, r17 data

ldi		r16,$0010	 ;location in EEPROM low byte of low pair
rcall	FetchEEPROM
sts		$0083,r17
sts		$0088,r17

ldi		r16,$0011	 ;high byte of low pair
rcall	FetchEEPROM
sts		$0084,r17
sts		$0089,r17

ldi		r16,$0012	;low byte of high pair
rcall	FetchEEPROM
sts		$0085,r17
sts		$008A,r17

ldi		r16,$0013	;high byte of high pair
rcall	FetchEEPROM
sts		$0086,r17
sts		$008B,r17

ldi		r16,$0014	;zero, 1= minus second or 2 = plus second
rcall	FetchEEPROM
sts		$0087,r17


;-------------------------------

ldi		r16,224
sts		$0068,r16
ldi		r16,1
sts		$0069,r16	;set for 8:00 a.m. time (480 seconds)
ldi		r16,1		;set for Sunday, day of week
sts		$006A,r16

;-------------------------------

sei		;On interrupts

sbr		r19,1	;Set r19,0(1), Delay Kill flag to ON

;--------------------------------------------------------

;FLAGS

;r19,0(1) = Kill delay flag 1=kill
;r19,1(2) = PM flag 1 = PM
;r19,2(4) = Double Flash for Beep/LED
;r19,3(8) = Timer time-up flag 1=time up
;r19,4(16) = Double flash for variable clock
;r19,5(32) = Turn on clock alarm
;r19,6(64) = newminute flag
;r19,7(128) = second counter flag

;r20,0(1) = Alarm time is now 1=yes
;r20,1(2) = Sonalert alarm on at clock alarm time 0=off
;r20,2(4) = Relay status flag  1 = ON
;r20,3(8) = Quick Set Alarm - One Time Only flag
;r20,4(16) = Timer running flag
;r20,5(32) = Beeper Alarm On/Off (0 = on, 1 = off)
;r20,6(64) = Repeat Last Timer time Flag
;r20,7(128) = Clock Correction Due flag

;r24,25 = Timer Count register pair


;-----------------------------------------

;Main Loop which jumps to procedures to accomplish tasks


MainLoopStall:

rcall	StallIfButtonDown


MainLoop:	;direct jump into Main Loop - no stall if switch is down

sbis	PIND,2		;skip if PIND,2 set - button up
rjmp	DecodeButtonPush


ML1:

lds		r16,$006b	  ;get seconds count
cpi		r16,0
brne	ML2

ldi		r16,15

rjmp	RedGreenWagR16	;long wag to signify it is 0 second Display Clock


ML2:	;Check for 30 seconds (clock display)

cpi		r16,30
brne	ML3

ldi		r16,7
rjmp	RedGreenWagR16


ML3:

sbrs	r20,0	;skip if r20,0(1) clock alarm now flag is set				
rjmp	MainLoop

rjmp	SoundAlarmOnRelay

;END of Loop.  All branches consist of jumps, not rcalls


;---------------------------------

;See if it is a Timer entry or to process Commands

MainLoopBypass:	;Used by display clock code for re-entry

DecodeButtonPush:

rcall	ButtonPush

mov		r16,r0  ;get number of code digits in entry
cpi		r16,5   ;is this valid code? r16 with 5 digits?   

;ALL codes start with DIT DAH
;  last remaining digits vary according to command

breq	DBcode	;further checking and decoding needed
rjmp	DBP1	;it is a TIMER entry - skip to timing


DBcode:	;Determines if it is a command or entry for timer
	;  Commands are then decoded

mov		r16,r1	;move r0 5 code digits into r16
andi	r16,24	;mask to save digit 4(8) and 5(16) 

cpi		r16,8	;check for DIT DAH (8)
breq	DBcode1
rjmp	DBP1	;this is a TIMER entry - skip to timing


DBcode1:		;It is a command entry

mov		r16,r1	;put low code byte into r16 for decoding

cpi		r16,8	;Toggle Relay
breq	DBcode1a
rjmp	DBcode2


DBcode1a:

rcall	ToggleRelay

sbrs	r20,2	;skip if r20,2(4) is set (relay on)
rjmp	DBcode1a1

rjmp	DBexit1


DBcode1a1:

rcall	DoubleShortBeep
rjmp	DBexit2


DBcode2:

cpi		r16,9	;Toggle Alarm on and off
breq	DBcode2a
rjmp	DBcode3


DBcode2a:

rcall	ToggleAlarm
rjmp	DBexit2


DBcode3:		;Set Alarm Time

cpi		r16,10
breq	DBcode3a
rjmp	DBcode4


DBcode3a:

rcall	SetAlarmTime
rjmp	DBexit1


DBcode4:		;Set clock

cpi		r16,11
breq	DBcode4a
rjmp	DBcode5


DBcode4a:

rcall	SetClock
rjmp	DBexit1


DBcode5:
cpi		r16,12	;Set Clock Correction
breq	DBcode5a
rjmp	DBcode6


DBcode5a:

rcall	SetClockCorrection
rjmp	DBexit1


DBcode6:

cpi		r16,13	;Once Only Quick Alarm Set
breq	DBcode6a
rjmp	DBcode7

DBcode6a:

rcall	QuickAlarm
rjmp	DBexit1


DBcode7:

cpi		r16,15
breq	DBcode7a
rcall	SingleLongBeep  ;No further valid codes - error
rjmp	DBExit2


DBcode7a:

rcall	ToggleBeep  ;Toggles clock alarm beep on/off
rjmp	DBexit2


DBexit1:

rcall	SingleShortBeep


DBexit2:	;error exit --- no command exectuted

rjmp	MainLoopStall	;abort entry - incorrect - 


;------------------------------------------------

DBP1:	;TIMER START - NOT A SUBROUTINE!

lds		r24,$0079
lds		r25,$007A	  ;put X60 total count into r24,25 pair

sts		$007C,r24  ;store current count into SRAM 
sts		$007D,r25  ;  for later use as repeat time
rjmp	DBP1b


DBP2:	;Used for REPEAT time for timer

lds		r24,$007C
lds		r25,$007D	;load repeat times from SRAM


DBP1b:

rcall	TurnRelayOn

sbr		r20,16  ;ON r20,4(16), Timer running flag

cbr		r19,8  ;clear TimeUp flag, r19,3(8)

cbr		r19,128  ;clear seconds counter flag, r19,7(128)


DBP1stall:

sbrs	r19,7	  ;skip if r19,7(128) seconds counter flag is set
rjmp	DBP1stall	  ;jump back until flag is set

lds		r16,$006B	
sts		$0062,r16	;put current seconds sample value into Seconds Ref for Timer

rcall	SingleShortBeep


DBP1a:

;TimerActiveSection

TimerActive:

sbis	PIND,2	;skip if PIND,2 is set (switch up)
rjmp	TAskip	;to decode button push

sbrc	r19,3	;skip if r19,3(8) clear - Time Up flag
rjmp	TimeUp

sbrs	r19,7	;skip if r19,7(128) is set (1 second flag)
rjmp	TimerActive

cbr		r19,128	;clear r19,7(128), 1 second flag

rcall	OneGreenFlash  ;Flash green LED periodically while in timing sequence
rjmp	TimerActive
	

TAskip:

rcall	ButtonPush

cpi		r30,2  ;low byte of total ---  jump out and cancel timer action

breq	TA1	;EXIT - button pushed twice exits

mov		r30,r24
mov		r31,r25  ;move remaining time into r30,31 for beep out of time left

rcall	BeepOutAllTime

rjmp	TimerActive


TA1:

rcall	TurnRelayOff
cbr		r20,16	      ;clear r20,4(16), Timer running flag
rcall	SingleShortBeep
rjmp	MainLoop


;TimeUpForCounter section


TimeUp:	;NOT A SUBROUTINE!

cbr		r19,8	;clear Time Up Flag, r19,3(8)
cbr		r20,16	;clear Timer running flag r20,4(16)

rcall	TurnRelayOff

lds		r18,$006B   ;grab reference second count and put in r18

ldi		r16,10	;for ten beeps of sonalert
rcall	BeepR16	;rcall r10 beeps

rcall	EveryMinuteUpCount  ;will beep out current "past time" count

sbrs	r20,6	;skip if r20,6(64), repeat timer time flag is set
rjmp	TimeUpEnd

cbr		r20,64	;clear r20,6(64) repeat timer time flag

rjmp	DBP2	;jump back to "repeat" timer section


TimeUpEnd:

rcall	SingleShortBeep

rjmp	MainLoop


;---------------

;All delays in milliseconds - .025 seconds - 025 (1/40th second)

UniversalDelay025:	;First part allows setting of delay which can be used
		;  by other delay routines.  Second part uses data, does the delay
		;  then returns. Values are for 32,768 crystal

		; If r19,0(1), kill delay flag, is set, delay is interruptible by
		;  touch plate entries

;There are 30 static counts, each loop takes 9 repeatable counts
;  GWBASIC program UNIDEL.BAS will calculate delay

sbrs	r19,0	;skip if r19,0(1), kill delay flag, is set
rjmp	UD025
sbis	PIND,2	;skip if PIND,2, switch is up
ret


UD025:

push	r16
push	r30
push	r31
ldi		r30,88
ldi		r31,0

;break point.  Following can be used to complete other delay routines

UD2:

sbrs	r19,0	;skip if r19,0(1), kill delay flag, is set
rjmp	UD2a

sbic	PIND,2	;skip if PIND,2, switch is down
rjmp	UD4
rjmp	UD3	;to exit


UD2a:

nop
nop


UD4:

sbiw	r30,1	;subtract 1 from r30,31 pair
brne	UD2


UD3:

pop		r31
pop		r30
pop		r16

ret


;-----------------------------------------


UniversalDelay050:

sbrs	r19,0	;skip if r19,0(1), kill delay flag, is set
rjmp	UD050
sbis	PIND,2	;skip if PIND,2, switch is up
ret


UD050:

push	r16
push	r30
push	r31
ldi		r30,178
ldi		r31,0

rjmp	UD2	;to finish delay


;---------------------------------------

UniversalDelay100:

sbrs	r19,0	;skip if r19,0(1), kill delay flag, is set
rjmp	UD100
sbis	PIND,2	;skip if PIND,2, switch is up
ret


UD100:

push	r16
push	r30
push	r31
ldi		r30,104
ldi		r31,1

rjmp	UD2	;to finish delay

;---------------------------------------

UniversalDelay250:

sbrs	r19,0	;skip if r19,0(1), kill delay flag, is set
rjmp	UD250
sbis	PIND,2	;skip if PIND,2, switch is up
ret


UD250:

push	r16
push	r30
push	r31
ldi		r30,185
ldi		r31,4

rjmp	UD2	;to finish delay


;------------------------------------------------

UniversalDelay500:

sbrs	r19,0	;skip if r19,0(1), kill delay flag, is set
rjmp	UD500
sbis	PIND,2	;skip if PIND,2, switch is up
ret


UD500:

push	r16
push	r30
push	r31
ldi		r30,25
ldi		r31,7

rjmp	UD2	;to finish delay


;---------------------------------------

UniversalDelay1000:	;One second delay 1000 milliseconds

sbrs	r19,0	;skip if r19,0(1), kill delay flag, is set
rjmp	UD1000
sbis	PIND,2	;skip if PIND,2, switch is up
ret


UD1000:

push	r16
push	r30
push	r31
ldi		r30,53
ldi		r31,14

rjmp	UD2	;to finish delay


;---------------------------------------

ClockCntDwnInt:   ;interrupt to update clock counters

push	r16
push	r17
push	r30
push	r31

in		r16,SREG	;save flags
sts		$0060,r16

sbr		r19,128	;set r19, bit 7(128) -- one second flag 
		;(set each second)


CC1b:

lds		r16,$006B	  ;load seconds counter $006B into r16


CorrectionsSub:	;When clock correction flag is set, correction is made at 5
		;  seconds into count.

cpi		r16,5	;is it 5 seconds into count?
brne	CSend

sbrs	r20,7	;skip if r20,7(128) is set -- Correction Due Flag
rjmp	CSend

cbr		r20,128	;clear r20,7(128) - correction due flag
lds		r17,$0087	;get clock correction type --- minus second, plus second

cpi	r17,1	
brne	CS1

dec		r16	;decrement second counter by one
rjmp	CSend


CS1:	;it was 2 or more in value

inc		r16  ;increment second counter by one


CSend:

;-------------------------------
;FOLLOWING section sets clock correction due flag when counter overflows

CCDEC:

lds		r17,$0087	 ;get correction change (no change, - 1 sec. or + 1 sec)
tst		r17
breq	CCDEC10  ;Exit if no clock correction is set

lds		r30,$0088
lds		r31,$0089	 ;get low byte of clock correction low pair, highest bytes

sbiw	r30,1
sts		$0088,r30
sts		$0089,r31
brcc	CCDEC10

lds		r30,$008A	
lds		r31,$008B
sbiw	r30,1
sts		$008A,r30
sts		$008B,r31
brcc	CCDEC10

lds		r17,$0083	 ;load from permanent store area, reload counter
sts		$0088,r17

lds		r17,$0084
sts		$0089,r17

lds		r17,$0085
sts		$008A,r17

lds		r17,$0086
sts		$008B,r17	;new count now reloaded

sbr		r20,128  ;set r20,7(128), clock correction due flag


CCDEC10:    ;end point


;----------------------------------

CC1bbb:

inc		r16	;step one count of SECONDS counter
sts		$006B,r16


;---------------------------------

CC1bb:

cpi		r16,60
breq	CCDI1

rjmp	CCDIExit	 ;return since no action required


CCDI1:

clr		r16	;zero r16 -- SECONDS counter
sts		$006B,r16

sbr		r19,64	;set R19 bit 6(64) -- the NEW ONE MINUTE flag


CCDI1Cntr:

lds		r16,$0068	  ;load low minutes byte into R16

inc		r16
sts		$0068,r16

brne	CCDI1a	;jump out if not zero (no overflow)

lds		r16,$0069	 ;high byte minutes
inc		r16	;increment high byte minutes
sts		$0069,r16

CCDI1a:

lds		r16,$0069	  ;check high byte if it is 5 (1440=  5 A0)
cpi		r16,5

breq	CCDI1b

rjmp	CCDIalarmExit


CCDI1b:			

lds		r16,$0068
cpi		r16,160	;1440 minutes reached -- it's now midnight

breq	CCDI1c

rjmp	CCDIalarmExit


CCDI1c:	;zero high and low bytes - it is midnight

clr		r16
sts		$0068,r16	 ;store away new start at zero of 1440 count
sts		$0069,r16

lds		r16,$006A	  ;get day of week and increment
inc		r16
sts		$006A,r16

cpi		r16,8

breq	CCDI1d


CCDI1e:

rjmp	CCDIalarmExit


CCDI1d:

ldi		r16,1	;load with 1 (Sunday)
sts		$006A,r16
rjmp	CCDI1e	;to continue


CCDIalarmExit:

AlarmCheck:  ;Checks for match of alarm time set to actual time if flag is set

sbrs	r19,5  ;skip if r19,5(32) is set (alarm flag on)
rjmp	CCDIExit

lds		r30,$0066	;low clock alarm byte
lds		r31,$0067	;high clock alarm byte

lds		r16,$0068
cp		r16,r30	;compare low time byte
brne	CCDIExit

lds		r16,$0069
cp		r16,r31	;compare high time byte
brne	CCDIExit

;Alarm Time is a match

sbr		r20,1	;ON clock alarm flag is now, r20,0(1)


CCDIExit:		;Exit this interrupt routine

;Following decrements Timer Counter, setting R19,3(8) when zero

lds		r17,$0062	;get variable seconds count - (r16 has current second)

lds		r16,$006b
cp		r16,r17	;compare current seconds to variable seconds count
brne	TimerDecrementEnd

sbiw	r24,1	;subtract one from r24,25 TIMER counter

brne	TimerDecrementEnd

sbr	r19,8	;set time up flag r19,3(8) ON


TimerDecrementEnd:

lds		r16,$0060	 ;SREG temporary store
out		SREG,r16	 ;restore SREG data

pop		r31
pop		r30
pop		r17
pop		r16

reti	;exit


;------------------------------------------------------------

StallButtonPush:	;stalls and waits for button push entry.

sei	;ON interrupts to keep timing during button push routines
	;using crystal clock


SBP1:

sbic	PIND,2	;skip if PIND,2, switch is down
rjmp	SBP1

rjmp	ButtonPush


;-----------------------------------------------------------

;SUBROUTINE

ButtonPush:  ;Takes in dit, dah, holddown, and coded input

push	r24
push	r25

cbr		r19,1  ;clear r19,0(1), kill delay flag -- turn it off


;r30,r31 (Z register pair contain total value of entry)

clr		r0	;clear number of bits in code
clr		r1	;clear lowest code count
clr		r2	;clear next code count
clr		r3	;clear next code count
clr		r4	;clear next code count (clear all code counts)
clr		r7	;HoldDown count
clr		r8	;dah count
clr		r9	;Dit Count
clr		r23	;universal counter
clr		r24
clr		r25

	;r30,31 has total count

B1:

inc		r23	;universal counter
rcall	UniversalDelay025	;1/40th second delay
cpi		r23,60	;delay before first holddown count - 40=one second 
brsh	B3		;branch if - it is a holddown count

sbis	PIND,2		;skip if switch is NOT down
rjmp	B1


B2:

cpi		r23,20	;compare r23 for 20 (1/2 second dah count)
brlo	B2b	;branch if lower (dit count)


B2a:	;it is dah count

inc		r8   	;increment dah count

sec		;set carry flag -- it is dah entry
rol		r1
rol		r2   ;rotate data into registers r1-r4
rol		r3
rol		r4

sbi	PortB,1 ;on red LED	- for common cathode bi-color LED

;cbi		PortB,1	;on red LED

rcall	UniversalDelay100	;.1 second delay

cbi		PortB,1		;off red LED - for common cathod bi-color LED

;sbi		PortB,1		;off red LED
clr		r23
inc		r0    ;increment number of code digits


B2d:	;no touch/switch up detection

sbis	PIND,2	;skip if switch is up
rjmp	B2e	;jump back - there is a new entry coming in

inc		r23
rcall	UniversalDelay025	;1/40th second delay
cpi		r23,70	;has switch been up 70 counts? 40=one second (1.75 		;  seconds)
brsh	B4Decode1
rjmp	B2d


B2e:

clr		r23
rjmp	B1


B2b:	;Dit count detected

inc		r9	;increment dit count

clc		;clear carry flag -- it is dit entry
rol		r1
rol		r2	;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortB,1	;On Red LED - for common cathode LED

;cbi		PortB,1	;On Red LED

rcall	UniversalDelay025

cbi	PortB,1	;Off Red Led - fir cinnin cathode LED

;sbi		PortB,1	;Off Red LED

inc		r0	;increment number of code digits (r0)
clr		r23	;clear r23
rjmp	B2d


B3:	;Holddown count has been detected

inc		r7  	;increment holddown count

sbi		PortB,1	;On Red LED - common cathode LED

;cbi		PortB,1	;On Red LED

rcall	UniversalDelay050

cbi		PortB,1	;Off Red LED for short blink - common cathode LED

;sbi		PortB,1	;Off Red LED for short blink

clr		r23	;reset r23 to zero count


B4:

inc		r23
rcall	UniversalDelay025
sbic	PIND,2	;skip if switch is down
rjmp	B4a	;jump out and look for next entry

cpi		r23,30	;pause between continuous hold down of switch
brlo	B4

rjmp	B3


B4a:

clr		r23
rjmp	B2d	;Check for exit or new entry



B4Decode1:

;This section finalizes.  It combines counts

clr		r30
clr		r31	;ready registers for input

mov		r16,r8	;move dah count in r8 into r16


B4comb1:

tst		r16
breq	B4comb2

adiw 	r30,10	;add value of 10 in r30,31
dec		r16			
rjmp	B4comb1


B4comb2:

add		r30,r9	;add value in r9 (dit count) to r30
brcs	B4comb2a
rjmp	Bholdown1


B4comb2a:

inc		r31  ;carry was set so increment higher byte


Bholdown1:

;Combined value now in Z register pair, r30,31

mov		r16,r7	;put hold down count into r16

Bholdown1b:

tst		r16	;test r7(r16), hold down count
breq	BPEnd
adiw	r30,50	;add fifty count to r30,31 word total
dec		r16
rjmp	Bholdown1b

;Complete combined total now in Z register pair


BPEnd:

;Following adds 10 to hold down count for W register pair, r24,25

mov		r16,r7	;move number of hold down counts into r16
mov		r24,r30
mov		r25,r31	;transfer total values in r30,31 pair to r24,25 pair


B60a:

tst		r16			
breq	BPEnd2	;r24,25 pair now has X60 values

adiw	r24,10
dec		r16
rjmp	B60a


BPEnd2:

;Store away in SRAM $00E7-00EB, Dit,Dah,HolddownCount,LSD Total, MSD Total

sts		$0070,r0	;number of code digits
sts		$0071,r1	;low byte of code digit
sts		$0072,r2	;next byte of code digit
sts		$0073,r3	;high byte of code digit

sts		$0074,r9	;number of dits
sts		$0075,r8	;number of dahs
sts		$0076,r7	;number of hold down counts
sts		$0077,r30	;LSD of total count
sts		$0078,r31	;MSD of total count

sts		$0079,r24
sts		$007A,r25	;store X60 total count

mov		r21,r30
dec		r21	;one less count in r30, LSD of total 		;count for Plus One Entries.

 ;r21 has one less count

pop		r25
pop		r24

sbr		r19,1	;set r19,0(1), kill delay flag back on

ret


;--------------------------------

SingleLongBeep:

cbi		PortB,3	;ON sonalert
rcall	UniversalDelay1000
sbi		PortB,3	;OFF sonalert
	
ret


;--------------------------------

DoubleShortBeep:

cbi		PortB,3	;ON sonalert
rcall	UniversalDelay025
sbi		PortB,3	;OFF sonalert
rcall	UniversalDelay100


SingleShortBeep:

cbi		PortB,3	;ON sonalert
rcall	UniversalDelay025
sbi		PortB,3	;OFF sonaler

rcall	UniversalDelay100

ret


;--------------------------------

SetClock:	 ;sets clock for day of week, AM/PM, hour, minutes
			;  24 hour time entry required
			;0 = midnight (1440 minute ounter)

;SPECIAL NOTE:  -- Unit Number, Hours, and Minutes are 
;  done by dit (1), dah(10), and hold down (50) for numbers to be entered.
;  a RED flash precedes all entries that use the dit/dah/hold down button push.
;  PLUS ONE entry system --- zero = one entry,  240=241 entry

ldi		r16,4	;set up for FOUR green flashes
rcall	GreenFlashR16  ;indicate that four entries are needed

rcall	StallButtonPush	
mov		r16,r21	;get one less than low byte total (r21) (plus one entry)
andi	r16,7	;trim off any excess minutes
mov		r18,r16	;save tempoorarily in r18

rcall	OneGreenFlash

rcall	StallButtonPush	;enter hours -- 24 hour format
			;  1 = 0,  2 = 1,  3 = 2,  etc.
clr		r26
clr		r27	;zero register pair


SC1a:

tst		r21	;one less (plus one entry) from ButtonPush (r21)
breq	SC1b

adiw	r26,60	;add 60 to change hours into minutes
dec		r21
brne	SC1a


SC1b:

rcall	OneGreenFlash	;signal another entry needed

rcall	StallButtonPush	;enter minutes
add		r26,r21		;add r21 to r26,27 pair
brcc	SC1d

inc		r27	;add one to r27 (carry)


SC1d:

;minute total now complete

rcall	OneGreenFlash  ;signal another entry needed (final entry)



SC1e:	;Stalls until button pushed

sbic	PIND,2	;skip if button is pushed
		;START clock upon last button push
rjmp	SC1e


SetEnd:

cli	;turn off global interrupts

sts		$006A,r18	  ;store day-of-week
sts		$0068,r26	  ;store low byte in $0068
sts		$0069,r27	  ;store high byte in $0069
clr		r16	  		  ;zero seconds counter
sts		$006B,r16	  ;store zero seconds

sei		;ON interrupts

SC1f:	;Stall if button down

rcall	StallIfButtonDown


SetEnd2:

ret

;--------------------------------------


SetClockCorrection:	;Sets one second speedup/slowdown at entered interval

;FIVE entries required.. Wait for green light flash before making next entry.

;First entry is 16,777,216 units (0-255).  Seond is 65,536 units (0-255),  
;Third is 256 units.  Fourth and final
;   is single units (0-255).  Fifth and last is whether you wish this to be a
;   (0) no change, (1) subtract second,  (2) add second.

;Keep in mind that ALL entries use PLUS ONE system.  That is if you wish to enter
;     a zero, you push the button once.  A four is entered by pushing the button
;     five times,  223 is entered by keying in 224, etc.

cli	;OFF interrupts during this session

ldi		R16,5	;prepare for five flashes - five entries
rcall	GreenFlashR16

rcall	StallButtonPush  ;Enter 16,777,216 units entry (0-255)
mov		r17,r21	 ;put data value in r17
sts		$0086,r17  ;store in permanent area of SRAM
sts		$008B,r17	 ;store in clock correction counter
ldi		r16,$0013	 ;EEPROM location
rcall	WriteEEPROM

rcall	OneGreenFlash  ;Alert that it is ready for another entry

rcall	StallButtonPush  ;Enter 65,536 units entry (0-255)
mov		r17,r21	 ;put data value in r17
sts		$0085,r17	 ;store in permanent area of SRAM
sts		$008A,r17	 ;store in clock correction counter
ldi		r16,$0012	 ;EEPROM location
rcall	WriteEEPROM

rcall	OneGreenFlash  ;Alert that it is ready for another entry

rcall	StallButtonPush  ;Enter 256 units entry (0-255)
mov		r17,r21	;put data value in r17
sts		$0084,r17 ;store in permanent area of SRAM
sts		$0089,r17	;store in clock correction counter
ldi		r16,$0011	;EEPROM location
rcall	WriteEEPROM

rcall	OneGreenFlash  ;Alert that it is ready for another entry

rcall	StallButtonPush	;Enter single units entry (0-255)
mov		r17,r21	;put data value in r17
sts		$0083,r17	;store in permanent area of SRAM
sts		$0088,r17	;store in clock correction counter
ldi		r16,$0010	;EEPROM location
rcall	WriteEEPROM

rcall	OneGreenFlash  ;Alert that it is ready for another entry

rcall	StallButtonPush  ;Enter 0 for NO correction, 1 for subtract one second
		          ;  2 for add one second
mov		r17,r21	;put data value in r17 (no correction, -1 sec, or +1 sec)
sts		$0087,r17	;store in permanent area of SRAM
ldi		r16,$0014	;EEPROM location
rcall	WriteEEPROM

sei	;ON interrupts

ret


;--------------------------

WriteEEPROM:	;r16 = address, r17 = data

cli	;OFF interrupts

sbic	EECR,EEPE
rjmp	WriteEEPROM

out		EEAR,r16 	;out address

out		EEDR,r17	         ;out data
sbi		EECR,EEMPE  ;out one to EEMPE
sbi		EECR,EEPE

sei	;ON interrupts
ret


;---------------------------

FetchEEPROM:  ;r16 = address,   r17 = data received

cli	;OFF interrupts

FetchEP:	;R16 data in, R19 data out


sbic	EECR,EEWE
rjmp	FetchEP

out		EEAR,r16
sbi		EECR,EERE

in		r17,EEDR

sei	;ON interrupts
ret


;------------------------------------------

BeepR16:	  ;Beeps number of times in r16

push	r16


BeepR16a:

sbis	PIND,2	;skip if PIND,2 set (switch is up)
rjmp	BR16end

cbi		PortB,3	;ON beep
rcall	UniversalDelay100	
sbi		PortB,3	;OFF beep
rcall	UniversalDelay250

dec		r16
brne	BeepR16a


BR16end:

pop		r16
ret


;---------------------------------------------

BicolorRG100R16:	;Yellow flsh (red+green)is seen
					;For number of times in r16
push	r30
push	r31


BRG100b:

sbis	PIND,2	;skip if PIND,2 set (switch is up)
rjmp	BRG100end

ldi		r30,153
ldi		r31,1


BRG100c:	  ;YELLOW display

sbi		PortB,1	;ON red - THIS SECTION FOR Common Cathode LED
cbi		PortB,1	;OFF red
sbi		PortB,2	;ON green
cbi		PortB,2	;OFF green


;cbi		PortB,1	;ON red
;sbi		PortB,1	;OFF red
;cbi		PortB,2	;ON green
;sbi		PortB,2	;OFF green

sbiw	r30,1
brne	BRG100c

rcall	UniversalDelay250
dec		r16
brne	BRG100b


BRG100End:

pop		r31
pop		r30

ret


;----------------------------------------

RedFlashR16:  ;Flashed RED LED number of times in r16

sbis	PIND,2	;skip if PIND,2 set (switch is up)
ret

sbi	PortB,1	;ON red LED - for common cathode LED


;cbi		PortB,1	;ON red LED

rcall	UniversalDelay100

cbi	PortB,1 ;OFF LED - for common cathode LED

;sbi		PortB,1	;OFF LED 

rcall	UniversalDelay250

dec		r16
brne	RedFlashR16

ret


;-------------------------------------

RedFlashClockR16:	;Flashed RED LED number of times in r16
		;Double flash when r19,2(4) is set

sbis	PIND,2	;skip if PIND,2 set (switch is up)
ret

sbrs	r19,2	;skip if r19,2(4), double flash is set
rjmp	RFC1

sbi	PortB,1	;ON red led - for common cathode LED

;cbi		PortB,1	;ON red LED

rcall	UniversalDelay100

cbi	PortB,1	;OFF LED - for common cathod LED

;sbi		PortB,1	;OFF LED

rcall	UniversalDelay050


RFC1:

sbi	PortB,1	;ON red LED - for common cathode LED

;cbi		PortB,1	;ON red LED

rcall	UniversalDelay100

cbi	PortB,1	;OFF LED - for common cathode LED
	
;sbi		PortB,1	;OFF LED

rcall	UniversalDelay250

dec		r16
brne	RedFlashClockR16

ret


;-------------------------------------

GreenFlashClockR16:  ;Flashed GREEN LED number of times in r16
		   ;Double flash when r19,2(4) is set


sbis	PIND,2	;skip if PIND,2 set (switch is up)
ret

sbrs	r19,2	;skip if r19,2(4) is set (double flash flag)
rjmp	GFC1

sbi	PortB,2	;ON green LED - for common cathode LED

;cbi		PortB,2	;ON green LED

rcall	UniversalDelay100

cbi	PortB,2	;OFF LED - for common cathod LED

;sbi		PortB,2	;OFF LED

rcall	UniversalDelay050


GFC1:

sbi	PortB,2	;ON green LED - for common cathode

;cbi		PortB,2	;ON green LED
rcall	UniversalDelay100

cbi		PortB,2	;OFF LED - for common cathode

;sbi		PortB,2	;OFF LED

rcall	UniversalDelay250

dec		r16
brne	GreenFlashClockR16

ret


;-------------------------------------

OneRedFlash:

push	r16
ldi		r16,1
rcall	RedFlashR16
pop		r16
	
ret


;--------------------------------------

GreenFlashR16:

sbis	PIND,2	;skip if PIND,2 set (switch is up)
ret

sbi	PortB,2	;ON green LED - common cathode LED

;cbi		PortB,2	;ON green LED

rcall	UniversalDelay100

cbi	PortB,2	;OFF LED - common cathode LED

;sbi		PortB,2	;OFF LED

rcall	UniversalDelay250

dec		r16
brne	GreenFlashR16

ret


;---------------------------------------------

OneGreenFlash:

push	r16
ldi		r16,1
rcall	GreenFlashR16
pop		r16
 
ret


;----------------------------------------------

SetAlarmTime:	;Set Clock Alarm Time

push	r26
push	r27

sbr		r19,32	;Set r19,5(32) On .... alarm is now on

ldi		r16,2	;two RED LED flashes (2 entries required)
rcall	RedFlashR16

clr		r26
clr		r27

rcall	StallButtonPush


SAT1:

tst		r21	;r21 has one count less than total (plus one entry system)
breq	SAT2

adiw	r26,60	;Add 60 minutes to r30,31
dec		r21

brne	SAT1


SAT2:

rcall	OneRedFlash	;One plus entry

rcall	StallButtonPush


SAT2a:

add		r26,r21
brcc	SAT3

inc		r27


SAT3:

sts		$0066,r26
sts		$0067,r27	;store away in SRAM

pop		r27
pop		r26

ret


;----------------------------------

ToggleRelay:

sbrs	r20,2	;skip if r20,2(4) is set (relay on)
rjmp	TurnRelayOn


TurnRelayOff:

cbr		r19,1	;OFF kill delay flag, r19,0(1)
cbr		r20,4	;clear r20,2(4) relay flag to off
sbi		PortB,5
rcall	UniversalDelay025
cbi		PortB,5	;finish pulse OFF relay
rjmp	TRend


TurnRelayOn:

cbr		r19,1	;OFF kill delay flag, r19,0(1)
sbr		r20,4	;set r20,2(4) flag to ON
sbi		PortB,4
rcall	UniversalDelay025
cbi		PortB,4	;finish ON pulse for relay


TRend:

sbr		r19,1	;ON kill delay flag

ret


;------------------------

;FOLLOWING NOT A SUBROUTINE!!!!


RedGreenWagR16:	;Wags Red and Green LEDs for start of Display Clock
		;R16 has number of wags -- Not a counting wag

sbis	PIND,2	;skip if PIND,2, switch is up
rjmp	MainLoopBypass

sbi	PortB,1	;ON green LED - common cathode LED

;cbi		PortB,1	;ON green LED

rcall	UniversalDelay025

cbi	PortB,1	;OFF green LED - common cathode LED

;sbi		PortB,1	;OFF green LED

rcall	UniversalDelay025

sbi	PortB,2	;ON geen LED - common cathode LED

;cbi		PortB,2	;ON green LED

rcall	UniversalDelay025

cbi		PortB,2	;OFF green LED - common cathode

;sbi		PortB,2	;OFF green LED

;cbi	PortB,2	;OFF red LED - common cathode LED

rcall	UniversalDelay025

dec		r16
brne	RedGreenWagR16

rcall	UniversalDelay1000


FlashOutDayOfWeek:

lds		r16,$006A		;get day of week


FODOW1:

rcall  	BiColorRG100R16
rcall	UniversalDelay1000	;flash out in yellow, day of week


;Flash Out of Time Section


TwelveHourConversion:  ;Converts 1440 minute count into hours, tens of minutes, and minutes
	 ;  setting PM flag r19,1(2) for 12 hour afternoon times

sbis	PIND,2	;skip if PIND,2 set - (switch is up)
rjmp	MainLoopBypass

cli		;OFF interrupt

lds		r30,$0068
lds		r31,$0069			;get current time in r30,31

sei							;ON interrupt

cpi		r31,2
brne	THC1

cpi		r30,208
brsh	THC5	;It is now 720 seconds into count or higher
rjmp	THC4	;It is now 0 to 719 seconds into count


THC1:

cpi		r31,3
brsh	THC5	;go to 720 or higher final


THC4:		;0-719 count FINAL

cbr		r19,2	;clear r19,1(2), PM flag (it is AM time)
rjmp	DisplayFlashout


THC5:	;720 count or higher FINAL

sbr		r19,2	;set r29,1(2), PM flag  (it is PM time)
ldi		r16,12


THC5a:

sbiw	r30,60
dec		r16
brne	THC5a	;subtract 720 from r30,31 pair

;r30 now has PM hours and minutes
;12 hour conversion complete = 0 - 719


;Display Clock Flashouts of Time Section using 0-12 hours

DisplayFlashout:

clr		r16	;clear counter, r16

tst		r31
brne	DF1

cpi		r30,60
brlo	DF2


DF1: 	;number is 60 or higher

sbis	PIND,2	;skip if PIND,2 is set (switch up)
rjmp	MainLoopBypass

sbiw	r30,60
brcs	DF3
inc		r16
rjmp	DF1


DF2:

ldi		r16,12	;set r16 to flash out 12 flashes (for noon or midnight)
rjmp	DF4


DF3:

adiw	r30,60	;restore count


DF4:	;flash out hours

rcall	TimeFlashOutR16	;HOURS flash out


DF5:

sbis	PIND,2	;skip in PIND,2 is set (switch up)
rjmp	MainLoopBypass

sbiw	r30,10	;subtract 10 minute segment from word
brcs	DF6

inc		r16
rjmp	DF5


DF6:

adiw	r30,10	;restore count
sbr		r19,4	;set double flash on, r19,2(4)

rcall	TimeFlashOutR16	;10 Minute Segment flashout

sbis	PIND,2	;skip in PIND,2 is set (switch up)
rjmp	MainLoopBypass


DF7:

sbis	PIND,2	;skip if PIND,2 is set (switch up)
rjmp	MainLoopBypass

sbiw	r30,1	;subtract one minute
brcs	DF8
inc		r16
rjmp	DF7


DF8:

rcall	TimeFlashOutR16	;Minutes flashout

rjmp	MainLoop


;----------------------------------

TimeFlashOutR16:	;flashes out time in red if r19,1(2) is set (PM)
		;  green if r19,1(2) is clear (AM)

sbis	PIND,2	;skip if PIND,2 set (switch is up)
ret

tst		r16
breq	TFOEnd

;Time flashed out number of times in r16

sbrs	r19,1	;skip if r19,1(2) is set (PM time)
rjmp	TFOam

;It is PM time flashout


TFOpm:

sbis	PIND,2	;skip if PIND,2 is set - switch up
rjmp	TFOEnd

rcall	RedFlashClockR16
rcall	UniversalDelay500  ;half second delay between flashes

rjmp	TFOend



;  NEXT --- AM Time is flashed out


TFOam:	;It is green time flash out

sbis	PIND,2	;skip if PIND,2 is set - switch up
rjmp	TFOEnd

rcall	GreenFlashClockR16
rcall	UniversalDelay500	;quarter second delay between flashes


TFOend:

cbr		r19,4	;clear r19,2(4), double flash/beep flag
rcall	UniversalDelay1000	;one second delay between hour, ten minute, and minute

ret


;-----------------------------------------

BeepOutTimeR16:	;Beeps out time in r30,31 pair

;Time beeped out number of times in r16

tst		r16
breq	BOTEnd

BOT1:

sbrs	r19,2	;skip if r19,2(4) set - Double Flash/Beep flag
rjmp	BOTb


BOT2:

sbis	PIND,2	;skip if PIND,2 set (switch is up)
ret

rcall	DoubleShortBeep
rcall	UniversalDelay500
dec		r16
brne	BOT2
cbr		r19,4	;clear r19,2(4), double flash flag
	
rjmp	BOTend


BOTb:

sbis	PIND,2	;skip if PIND,2 set (switch is up)
ret

rcall	SingleShortBeep
rcall	UniversalDelay500	;quarter second delay between beeps
dec		r16
brne	BOTb


BOTend:

rcall	StallIfButtonDown

ret	;Time is beeped out for r16


;-----------------------------------------

BeepOutAllTime:	;beeps out hours, ten minute segments, and minutes
		;  R30,31 has values to be beeped out

push	r28
push	r29

ldi		r16,255	;max in r16 (zero count -1)


BAOT1:

mov		r28,r30
mov		r29,r31	;save temporarily

inc		r16	;HOURS

sbiw	r30,60	;subtract 60 min from r30
brcc	BAOT1

rcall	BeepOutTimeR16

ldi		r16,255

mov		r30,r28
mov		r31,r29			;restore old count


BAOT2:

mov		r28,r30
mov		r29,r31			;save updated count

inc	r16	;10 MINUTES

sbiw	r30,10
brcc	BAOT2

sbr		r19,4	;set double flash register r19,2(4)
rcall	BeepOutTimeR16
cbr		r19,4	;clear double flash
ldi		r16,255

mov		r30,r28
mov		r31,r29	;restore old value


BAOT3:

mov		r28,r30
mov		r29,r31	;save updated count

inc		r16

sbiw	r30,1	;ONE MINUTE
brcc	BAOT3

rcall	BeepOutTimeR16

pop		r29
pop		r28

ret


;---------------------------------------

ToggleAlarm:  ;toggles alarm flag on and off, r19,5(32) 1 = on

sbrs	r19,5	;skip if r19,5(32) is set (alarm on)
rjmp	Commands11a

cbr		r19,32	;clear r19,5(32) flag -- off alarm
rcall	DoubleShortBeep

ret


Commands11a:

sbr		r19,32	;set r19,5(32) -- turn alarm on
rcall	SingleShortBeep

ret


;---------------------------------

EveryMinuteUpcount:  ;Used for Timer up and Clock alarm

push	r26
push	r27

sbis	PIND,2	;skip if PIND,2 is set - button up
rjmp	EMUend

clr		r26
clr		r27	;clear r26,27 pair (up counter)


EMU3:

sbis	PIND,2	;skip if PIND,2 is set - button up
rjmp	EMUend


sbrs	r19,7		;skip if r19,7(128) one second flag is set
rjmp	EMU3a

rcall	OneRedFlash
cbr		r19,128		;clear r19,7(128) one second flag bit


EMU3a:

lds		r16,$006B	  ;get current second and compare against reference second in r18
cp		r16,r18
brne	EMU3

adiw	r26,1	;increment minute count by one in r26,27
mov		r30,r26
mov		r31,r27	;move into r30,31 for Beep Out of Time

rcall	BeepOutAllTime
rcall	UniversalDelay1000	;delay one second

cpi		r26,61	;exit after exceeding one hour
brlo	EMU3
rjmp	EMUend


EMUend:	;EXIT

pop		r27
pop		r26

rcall	ButtonPush  ;check for repeat of last timer time
cpi		r30,2	   ;two pushes for repeat of last time
brne	EMU4

sbr		R20,64	;set Repeat Same Timer Time flag r20,6(64)

ret


EMU4:

rcall	SingleShortBeep

ret


;----------------------------------------------

SoundAlarmOnRelay:  ;Alarm time routine -- turns on relay and sounds alarm with
		   ; reminders every minute.  After one hour, it turns off.
		   ; (Alarm Time Set Only)

cbr		r20,1	;clear alarm on flag, r20,0(1)

rcall	TurnRelayOn

sbrs	r20,5	;skip if r20,5(32) set -- 1 = silence, 0 = sound alarm
rjmp	SAOR1a

ldi		r30,16	;load counter for 60 minutes on time (3600 seconds)
ldi		r31,14


SAOR1aa:  ;SILENCE section - wag of LEDs signal it is in this segement

sbis	PIND,2	;skip if PIND,2 is set - button up
rjmp	SAOR2b	;jump out

sbrs	r19,7	;skip if r19,7(128) set - second counter flag
rjmp	SAOR1aa

cbr		r19,128	;clear r19,7(128) - interrupt second counter flag
ldi		r16,2
rcall	GreenFlashR16  ;two flashes of green

sbiw	r30,1	;subtract one second from total time
brcs	SAOR2b	;jump out
rjmp	SAOR1aa	  ;recycle - time not up yet


SAOR1a:

ldi		r16,15	;sound fifteen beeps on sonalert
rcall	BeepR16

cbr		r19,128	;clear r19,7(128), new second flag


SAOR1:

sbis	PIND,2
rjmp	SAOR2b

sbrs	r19,7	;skip if r19,7(128) is set (delay until new second occurs)
rjmp	SAOR1

lds		r18,$006B	  ;grab current second and put in r18
				      ;EveryMinuteUpCount requires reference second in r18
rcall	EveryMinuteUpCount  ;stays in this subroutine until button is pushed
                      ;  Beeps out accumulated time every minute
			  	      ;  When button pushed, relay is turned off

SAOR2b:

rcall	TurnRelayOff

sbrs	r20,3	;skip if r20,3(8) set - One time only flag
rjmp	SAOR2

cbr		r20,8	;clear r20,3(8) - One time only flag
cbr		r19,32	;clear r19,5(32) - Clock alarm on flag


SAOR2:

rcall	SingleShortBeep

rjmp	MainLoopStall


;---------------------------------------------

QuickAlarm:  ;alarm entry for quarter hours past midnight
	     ;  --- a one time only alarm set

rcall	OneRedFlash
rcall	StallButtonPush

mov	r16,r21	;move the one less entry to r30 (plus one entry system)

clr		r31	;clear high and low bytes of r30,31 pair 		 
clr		r30	; to accept 15X multiplication


QAlarm1:

tst		r16
breq	QAlarmEnd

adiw	r30,15
dec		r16
rjmp	QAlarm1


QAlarmEnd:

sbr		r20,8	;set r20,3(8) - Quick alarm one time only flag
sbr		r19,32	;set r19,5(32), On alarm flag
sts		$0066,r30
sts		$0067,r31	;store in alarm SRAM location

ret


;----------------------------------------------

ToggleBeep:	;Toggles alarm beep on/off

sbrs	r20,5  ;skip if r20,5(32) set - Alarm beep flag
rjmp	TBeep1

rcall	SingleShortBeep
cbr		r20,32	;clear r20,5(32) - beep flag
ret


TBeep1:

sbr		r20,32	;set r20,5(32) - beep flag
rcall	DoubleShortBeep
ret


;-------------------------------------------------

StallIfButtonDown:

sbis	PIND,2	;skip if PIND,2 set - button up
rjmp	StallIfButtonDown

ret


;--------------------------------------------------

